;+
; NAME:
;   Main.pro 
;
; PURPOSE:
;
; This program reads AIRS and Cloudsat data, and then colocates the desired fields.  The output is a .dat file
; that is used in the other directory to plot the data.
;
; AUTHOR:
;
;   Brian H. Kahn
;   Jet Propulsion Laboratory
;   4800 Oak Grove Drive
;   MS 169-237
;   Pasadean, CA  91109
;   Tel. 818-393-0676
;
; REQUIREMENTS
;
;   * Need AIRS header files (end in .hdr), the L2 Standard file, and the Cloudsat GEOPROF file.
;
; KNOWN ISSUES:
;
;   * No functionality (yet) to read Cloudsat granules that span across two contiguous days (one every ~16 Cloudsat granules)
;   * Paths to the Cloudsat and AIRS data are hardwired in.
;   * This code is provided as is.  As the author obtains time, he will continue to modify and improve this routine
;
; SOME QUIRKY OPERATIONS
;
;   * The AIRS cloud height is calculated from the AIRS geopotential and cloud top pressure fields.  A simple linear
;     interpolation between the two nearest AIRS standard geopotential levels is done to convert the cloud top pressure
;     to a height.  It is not perfect, but its a close, approximate, quick value, for now.  This will be improved.
;   * Sometimes AIRS does not report a geopotential value because of the way that the retrieval is done.  So, in these cases,
;     the nearest valid geopotential field is used.  If there are two or more that are the same distance away, the first
;     match is used.
;   * Cloudsat is assumed to trail Aqua by 54 seconds.  +/- 20 seconds of leeway is coded in to find nearest matches.  By
;     picking a window to match data with, a lot of time is saved in reading and matching data.
;
; EXAMPLE:
;   
;  @Compile
;  main, 2006, 7, 20, 1213, 1213
;        [year], [month], [day], [start cloudsat granule], [end cloudsat granule]
;  More than one Cloudsat granule can be processed (hence, the start and end).  But, there is no functionality (yet) to go
;  from one day to another.
;
; MODIFICATION HISTORY:
;
;   Version 1.0, written August 31st, 2006
;

PRO  Main, year, month, day, start_cloudsat, end_cloudsat

for iii = start_cloudsat, end_cloudsat, 1  do begin
 
   cs_granule = string(iii,format='(i5.5)') 
; Cloudsat L2 cloud boundary granule (1 orbit), Jay Mace owns this product
  pattern = '/home/kahn/Cloudsat/?????????????_'+cs_granule+'_CS_2B-GEOPROF_GRANULE_P_R??_E??.hdf'   

; Read Cloudsat granule
  read_Cloudsat_geoprof, pattern, Profile_time, UTC_start, TAI_start, Latitude, Longitude, Height, $
                         Spacecraft_altitude, DEM_elevation, RayHeader_RangeBinSize, Data_quality, Data_status,  $
                         Data_targetID, SurfaceHeightBin, CPR_Cloud_mask, Gaseous_Attenuation, Radar_Reflectivity, $
                         MODIS_Cloud_Fraction, MODIS_scene_char, MODIS_scene_var, CPR_Echo_Top, sem_NoiseFloor

  max_Cloudsat = n_elements(Latitude)

; Loop through AIRS header files
  kk1 = 0
  kk2 = 0
  num_airs_gran = 240
  tai_start_time = dblarr(num_airs_gran)
  tai_stop_time = dblarr(num_airs_gran)

  for i = 1, 240  do begin

       if size(month,/type) ne 7 then  month = string(month,format='(i2.2)') 
       if size(day,/type) ne 7 then   day = string(day,format='(i2.2)')
       if size(year,/type) ne 7 then   year = string(year,format='(i4.4)') 
       if size(i,/type) ne 7 then   granule = string(i,format='(i3.3)')

       pattern_header = '/home/kahn/Cloudsat/AIRS/AIRS.'+year+'.'+month+'.'+day+'.'+granule+'.L2.RetStd.v4.0.*.????????????.hdf.hdr'

       header = ''
       openr, u, pattern_header, /get
       for j = 0, 96  do begin
            readf, u, header
            if(j eq 85) then begin
                 equal_sign = strpos(header,"=")
                 tai_start_time[kk1] = strmid(header,equal_sign+1,14)
                 kk1 = kk1 + 1
            endif
            if(j eq 86) then begin
                 equal_sign = strpos(header,"=")
                 tai_stop_time[kk2] = strmid(header,equal_sign+1,14)
                 kk2 = kk2 + 1
            endif
       endfor
       free_lun, u

       lower_bound_tai_airs = TAI_start - 54.0D - 20.0D
       upper_bound_tai_airs = TAI_start - 54.0D + 20.0D

       num_Cloudsat_profiles = n_elements(Profile_time)
       print, TAI_start, Profile_time[num_Cloudsat_profiles-1], tai_start_time[kk1-1], tai_stop_time[kk2-1],  $
              tai_start_time[kk1-1]-TAI_start, format='(1x,4f14.3,2x,f14.7)

; Test for first AIRS granule
       if( (lower_bound_tai_airs lt tai_stop_time[kk1-1]) and (upper_bound_tai_airs gt tai_start_time[kk1-1])) then begin
            first_airs_granule = i
            print, first_airs_granule, lower_bound_tai_airs, upper_bound_tai_airs, tai_start_time[kk1-1], tai_stop_time[kk1-1], format='(1x,i5,4f14.3)
       endif

; Test for last AIRS granule 
       if( (lower_bound_tai_airs+Profile_time[num_Cloudsat_profiles-1] lt tai_stop_time[kk1-1]) and  $
           (upper_bound_tai_airs+Profile_time[num_Cloudsat_profiles-1] gt tai_start_time[kk1-1])) then begin
            last_airs_granule = i
            print, last_airs_granule, lower_bound_tai_airs, upper_bound_tai_airs, tai_start_time[kk1-1], tai_stop_time[kk1-1], format='(1x,i5,4f14.3)
       endif

  endfor

; Loop through AIRS granules
  latbig = [-77777.0D]
  lonbig = [-77777.0D]
  taibig = [-77777.0D]
  fp_index_big = [-77777.0D]
  line_index_big = [-77777.0D]
  gran_index_big = [-77777.0D]
  PCldTopUpperbig = [-77777.0D]
  PCldTopLowerbig = [-77777.0D] 
  ZCldTopUpperbig = [-77777.0D]
  ZCldTopLowerbig = [-77777.0D]

  for i = first_airs_granule, last_airs_granule   do begin

       if size(month,/type) ne 7 then  month = string(month,format='(i2.2)')
       if size(day,/type) ne 7 then   day = string(day,format='(i2.2)')
       if size(year,/type) ne 7 then   year = string(year,format='(i4.4)')
       if size(i,/type) ne 7 then   granule = string(i,format='(i3.3)')
 
       pattern2 = '/home/kahn/Cloudsat/AIRS/' + $
;       pattern2 = '/archive/AIRSOps/airs/gdaac/c3_v4/'+year+'/'+month+'/'+day+'/airx2ret/' + $
                  'AIRS.'+year+'.'+month+'.'+day+'.'+granule+'.L2.RetStd.v4.*.????????????.hdf'

       read_swath_l2, pattern2, numfp2, numline2, tai, lat, lon, pres, tair, h2o, ozo, scan_node_type=scan_node_type, start_Time=start_Time, $
                      retqa=retqa, TSurface=TSurface, $
                      TSurfErr=TSurfErr, CldFrc=CldFrc, PCldTop=PCldTop, TCldTop=TCldTop, TotH2OStd=TotH2OStd, totH2OErr=totH2OErr, $
                      CloudFrcErr=CloudFrcErr, PCloudTopErr=PCloudTopErr, TCloudTopErr=TCloudTopErr, TSAir=TSAir, GP_Height=GP_Height

       lower_pres_levels = pres[0:n_elements(pres)-2]
       upper_pres_levels = pres[1:n_elements(pres)-1]

       ZCldTopUpper = fltarr(numfp2,numline2)
       ZCldTopLower = fltarr(numfp2,numline2)
       for j = 0, numfp2-1  do begin 
            for k = 0, numline2-1  do begin 

;  If GP_Height is a bad retrieval, search for nearest neighbor with good retrieval                 
;  Find distances between different AMSU FOVs
                 if(GP_Height(15,j,k) eq -9999) then begin

                      arc_dist = fltarr(numfp2,numline2)
                      for l = 0, numfp2-1  do begin
                           for m = 0, numline2-1  do begin
                                arc_dist(l,m) = Map_2points(lon[l,m], lat[l,m], lon[j,k], lat[j,k], /meters)
                           endfor
                      endfor

                      loc_index = where( (arc_dist gt 1000.) and (GP_Height[15,*,*] ne -9999) )
                      min_arc_dist = min(arc_dist(loc_index))
                      loc_index2 = where(arc_dist eq min_arc_dist)
                      scanline = loc_index2/30
                      footprint = loc_index2 mod 30
                      if(n_elements(footprint) eq 1) then begin
                           print, j, k, GP_Height(15,j,k), GP_Height(15,footprint,scanline), lat[footprint,scanline], $
                                lat[j,k], lon[footprint,scanline], lon[j,k],  $
                                arc_dist(footprint,scanline), format='(1x, 2i5, 7f15.3)'
                           GP_Height(*,j,k) = GP_Height(*,footprint,scanline)
                      endif
                      if(n_elements(footprint) gt 1) then begin
;                           print, j, k, GP_Height(15,j,k), GP_Height(15,footprint[0],scanline[0]), lat[footprint[0],scanline[0]], $
;                                lat[j,k], lon[footprint[0],scanline[0]], lon[j,k],  $
;                                arc_dist(footprint[0],scanline[0]), format='(1x, 2i5, 7f15.3)'
                           GP_Height(*,j,k) = GP_Height(*,footprint[0],scanline[0])
                      endif
                 endif

                 test = where( (lower_pres_levels ge PCldTop[0,j,k]) and (upper_pres_levels le PCldTop[0,j,k]) )
                 if( (n_elements(test) eq 1) and (test[0] ne -1) ) then begin
                      pres_scalar = (PCldTop[0,j,k]-upper_pres_levels[test])/(lower_pres_levels[test]-upper_pres_levels[test])
                      ZCldTopUpper[j,k] = GP_Height[test+1,j,k] - (GP_Height[test+1,j,k]-GP_Height[test,j,k])*pres_scalar
;                      print, j, k, test, n_elements(test), PCldTop[0,j,k], lower_pres_levels[test], upper_pres_levels[test]
                 endif
                 test2 = where( (lower_pres_levels ge PCldTop[1,j,k]) and (upper_pres_levels le PCldTop[1,j,k]) )
                 if( (n_elements(test2) eq 1) and (test2[0] ne -1) ) then begin
                      pres_scalar2 = (PCldTop[1,j,k]-upper_pres_levels[test2])/(lower_pres_levels[test2]-upper_pres_levels[test2])
                      ZCldTopLower[j,k] = GP_Height[test2+1,j,k] - (GP_Height[test2+1,j,k]-GP_Height[test2,j,k])*pres_scalar2
;                      print, j, k, test2, n_elements(test2), PCldTop[1,j,k], lower_pres_levels[test2], upper_pres_levels[test2]
                 endif
                 if( (n_elements(test) eq 1) and (test[0] eq -1) ) then begin
                      ZCldTopUpper[j,k] = -9999.
                 endif
                 if( (n_elements(test2) eq 1) and (test2[0] eq -1) ) then begin
                      ZCldTopLower[j,k] = -9999. 
                 endif
                 if(n_elements(test) gt 1) then begin
                      if(upper_pres_levels[test[0]] eq PCldTop[0,j,k]) then  ZCldTopUpper[j,k] = GP_Height[test[0],j,k]
                      if(lower_pres_levels[test[0]] eq PCldTop[0,j,k]) then  ZCldTopUpper[j,k] = GP_Height[test[0],j,k]
                 endif
                 if(n_elements(test2) gt 1) then begin
                      if(upper_pres_levels[test2[0]] eq PCldTop[1,j,k]) then  ZCldTopLower[j,k] = GP_Height[test2[0],j,k]
                      if(lower_pres_levels[test2[0]] eq PCldTop[1,j,k]) then  ZCldTopLower[j,k] = GP_Height[test2[0],j,k]
                 endif
            endfor
       endfor

; Make a fp and scan index array, as well as a granule number array
       fp_index = intarr(numfp2,numline2)
       line_index = intarr(numfp2,numline2)
       gran_index = intarr(numfp2,numline2)
       for j = 0, numfp2-1  do begin
            for k = 0, numline2-1  do begin
                 fp_index[j,k] = j 
                 line_index[j,k] = k 
                 gran_index[j,k] = i 
            endfor
       endfor

; Make 2-D cloud top pressure arrays
       PCldTopUpper = fltarr(numfp2,numline2)
       PCldTopLower = fltarr(numfp2,numline2)
       for k = 0, numfp2-1 do begin
            for l = 0, numline2-1 do begin
                 PCldTopUpper[k,l] = PCldTop[0,k,l]
                 PCldTopLower[k,l] = PCldTop[1,k,l]
            endfor
       endfor

; Concatenate AIRS arrays for each granule into single ones for entire Cloudsat granule (orbit)
       if(latbig[0] eq -77777.0D) then begin
            latbig = lat
            lonbig = lon
            taibig = tai
            fp_index_big = fp_index
            line_index_big = line_index
            gran_index_big = gran_index
            PCldTopUpperbig = PCldTopUpper
            PCldTopLowerbig = PCldTopLower
            ZCldTopUpperbig = ZCldTopUpper
            ZCldTopLowerbig = ZCldTopLower
       endif else begin
            GlueArray, latbig, lat, latbig
            GlueArray, lonbig, lon, lonbig
            GlueArray, taibig, tai, taibig
            GlueArray, fp_index_big, fp_index, fp_index_big
            GlueArray, line_index_big, line_index, line_index_big
            GlueArray, gran_index_big, gran_index, gran_index_big
            GlueArray, PCldTopUpperbig, PCldTopUpper, PCldTopUpperbig
            GlueArray, PCldTopLowerbig, PCldTopLower, PCldTopLowerbig
            GlueArray, ZCldTopUpperbig, ZCldTopUpper, ZCldTopUpperbig
            GlueArray, ZCldTopLowerbig, ZCldTopLower, ZCldTopLowerbig
       endelse

  endfor

;  distance = latbig
  arc_dist = latbig
  for i = 0., num_Cloudsat_profiles-1.0  do begin 

       match = where( (taibig lt (upper_bound_tai_airs[0]+Profile_time[i])) and (taibig ge (lower_bound_tai_airs[0]+Profile_time[i])), cnt )

       for idist = min(match), max(match)  do begin
;            distance(idist) = sqrt( (lat(idist)-Latitude[i])^2. + (lon(idist)-Longitude[i])^2. )
            arc_dist(idist) = Map_2points(lonbig(idist), latbig(idist), Longitude[i], Latitude[i], /meters)
       endfor

       first_index = min(match)
       last_index = max(match)
;  arc_dist is more accurate for finding the closest one
;  map_2points is an IDL function

       temp1 = arc_dist[first_index:last_index]
       temp2 = min(arc_dist[first_index:last_index])
       loc_index = where(temp1 eq temp2)
       loc_index = loc_index+first_index

       print, fp_index_big[loc_index], line_index_big[loc_index], gran_index_big[loc_index], Latitude[i], latbig[loc_index],  $
              Longitude[i], lonbig[loc_index],  $
              PCldTopUpperbig[loc_index], PCldTopLowerbig[loc_index], ZCldTopUpperbig[loc_index], ZCldTopLowerbig[loc_index], $
              format='(1x,3i6,8f12.4)'

       temp5 = strcompress(string(first_airs_granule,format='(i3.3)'), /remove_all)
       temp6 = strcompress(string(last_airs_granule,format='(i3.3)'), /remove_all)
       openw, 1, 'lat_lon_match_'+month+'_'+day+'_'+year+'_'+temp5+'_'+temp6+'_'+cs_granule+'.dat', /append
       printf, 1, fp_index_big[loc_index], line_index_big[loc_index], gran_index_big[loc_index], Latitude[i], latbig[loc_index],  $
                  Longitude[i], lonbig[loc_index],  $
                  PCldTopUpperbig[loc_index], PCldTopLowerbig[loc_index], ZCldTopUpperbig[loc_index], ZCldTopLowerbig[loc_index], $
                  format='(1x,3i6,8f12.4)'
       if(i eq num_Cloudsat_profiles-1.0) then  printf, 1, num_Cloudsat_profiles

       close, 1

  endfor

endfor

stop
end
